@@ -152,6 +152,18 @@ def create_parser(parser, use_gooey=True):
         help="Particles with a distance less than this value (in pixel) will be removed. This option should not be used in filament mode.",
     )
 
+    optional_group.add_argument(
+        "--minsize",
+        type=int,
+        help="Particles with a estimated diameter less than this value (in pixel) will be removed. This option typically is only useful for the general model.",
+    )
+
+    optional_group.add_argument(
+        "--maxsize",
+        type=int,
+        help="Particles with a estimated diameter greather than this value (in pixel) will be removed.  This option typically is only useful for the general model.",
+    )
+
     optional_group.add_argument(
         "-pbs",
         "--prediction_batch_size",
@@ -296,7 +308,6 @@ def get_parser():
 
 
 def _main_():
-    import sys
 
     # if sys.argv[1] == "gui":
 
@@ -324,6 +335,7 @@ def main(args=None):
         pass
     import cryolo.utils as util
     util.check_for_updates()
+
     print("START METHOD:", multiprocessing.get_start_method())
 
     if args is None:
@@ -390,7 +402,8 @@ def main(args=None):
     if args.output is not None:
         outdir = str(args.output)
     write_empty = args.write_empty
-
+    min_size = args.minsize
+    max_size = args.maxsize
     num_cpus = int(multiprocessing.cpu_count()/2)
     if args.num_cpu != -1:
         num_cpus = args.num_cpu
@@ -409,6 +422,19 @@ def main(args=None):
                 new_input_size,
             )
             config["model"]["input_size"] = new_input_size
+    # Setup log dir
+    if "other" in config and "log_path" in config["other"]:
+        log_path = config["other"]["log_path"]
+    else:
+        log_path = "logs/"
+    if not os.path.exists(log_path):
+        os.makedirs(log_path)
+
+    # Write command to disk
+    import time
+    timestr = time.strftime("%Y%m%d-%H%M%S")
+    utils.write_command(os.path.join(log_path,"cmdlogs/", "command_predict_" + timestr + ".txt"),
+                        "cryolo_predict.py " + ' '.join(sys.argv[1:]))
 
     filament_mode = args.filament
     if filament_mode:
@@ -433,8 +459,10 @@ def main(args=None):
     overlap_patches = 0
     if "overlap_patches" in config["model"]:
         overlap_patches = int(config["model"]["overlap_patches"])
-    elif not len(config["model"]["anchors"]) > 2:
-        overlap_patches = config["model"]["anchors"][0]
+    elif "anchors" in config["model"]:
+        if not len(config["model"]["anchors"]) > 2:
+            overlap_patches = config["model"]["anchors"][0]
+
 
     write_direct = True
     if filament_mode:
@@ -458,6 +486,8 @@ def main(args=None):
             min_distance=min_distance,
             outdir=outdir,
             monitor=monitor,
+            min_size=min_size,
+            max_size=max_size
         )
     except tf.errors.ResourceExhaustedError:
         print("############################")
@@ -467,8 +497,8 @@ def main(args=None):
 
     export_size = config_tools.get_box_size(config)[0]
     if write_direct == False:
-        del_not_fully_immersed, del_min_distance = picking_postprocessing(
-            picking_results, export_size, min_distance
+        del_not_fully_immersed, del_min_distance, del_min_size  = picking_postprocessing(
+            picking_results, export_size, min_distance, min_size=min_size,max_size=max_size
         )
 
         if min_distance > 0:
@@ -484,6 +514,13 @@ def main(args=None):
             "particles as they were not fully immersed in the micrograph",
         )
 
+        print(
+            "Deleted",
+            del_min_size,
+            "particles as they were not out of specified size range",
+        )
+
+    write_size_distribution_to_disk(picking_results,os.path.join(outdir,"DISTR"))
     ###############################
     #   Filament Post Processing
     ###############################
@@ -619,7 +656,7 @@ def main(args=None):
                     if len(filaments) >= 1:
                         picking_result_subset[index_subset]["filaments"] = filaments
                     else:
-                        picking_result_subset[index_subset]["filaments"] = None
+                        picking_result_subset[index_subset]["filaments"] = []
 
                 print("Total number of filaments picked so far: ", picked_filaments)
                 process_counter += 1
@@ -636,7 +673,7 @@ def main(args=None):
 
             for result in picking_result_subset:
 
-                if (result["filaments"] is not None) or write_empty:
+                if result["filaments"] or write_empty:
                     pth = result["pth"]
                     eman_helix_segmented_path = pth
                     eman_start_end = pth
@@ -685,6 +722,114 @@ def main(args=None):
             prediction_result_to_disk(outdir, picking_results)
 
 
+def write_size_distribution_to_disk(picking_results, output_folder = ""):
+
+    if not os.path.exists(output_folder):
+        os.makedirs(output_folder)
+    estimated_size = []
+    confidence = []
+    for box_to_write in picking_results:
+        boxes = box_to_write["boxes_unfiltered"]
+        for box in boxes:
+            est_width = box.meta["boxsize_estimated"][0]
+            est_height = box.meta["boxsize_estimated"][1]
+            estimated_size.append((est_height+est_width)//2)
+            confidence.append(box.c)
+    #################################
+    # Size distribution histogram
+    #################################
+    est_size_mean = int(np.mean(estimated_size))
+    est_size_sd = int(np.std(estimated_size))
+    est_size_25q = int(np.percentile(estimated_size, 25))
+    est_size_50q = int(np.percentile(estimated_size, 50))
+    est_size_75q = int(np.percentile(estimated_size, 75))
+    print("#####################################")
+    print("")
+    print("## Particle diameter distribution ##")
+    print("MEAN:", est_size_mean , "px")
+    print("SD:", est_size_sd ,"px")
+    print("25%-Quantile:", est_size_25q,"px")
+    print("50%-Quantile:", est_size_50q,"px")
+    print("75%-Quantile:", est_size_75q,"px")
+
+
+
+    import matplotlib.pyplot as pl
+    import matplotlib as mpl
+    mpl.rcParams["figure.dpi"] = 200
+    mpl.rcParams.update({"font.size": 7})
+    width = max(10, int((np.max(estimated_size)-np.min(estimated_size))/10))
+    pl.hist(estimated_size, bins=width)
+    pl.title("Particle diameter distribution")
+    pl.xlabel("Partilce diameter [px] (Bin size: "+str(width)+"px )")
+    pl.ylabel("Count")
+
+    import time
+    timestr = time.strftime("%Y%m%d-%H%M%S")
+    import csv
+
+    output_path_distr_raw_txt = os.path.join(output_folder, "size_distribution_raw_" + timestr + ".txt")
+    np.savetxt(output_path_distr_raw_txt,estimated_size,fmt="%d")
+    output_path_distr_txt = os.path.join(output_folder, "size_distribution_summary_" + timestr + ".txt")
+    with open(output_path_distr_txt, 'w', newline='') as file:
+        writer = csv.writer(file)
+        writer.writerow(["Metric", "Value"])
+        writer.writerow(["MEAN", est_size_mean])
+        writer.writerow(["SD", est_size_sd])
+        writer.writerow(["Q25", est_size_25q])
+        writer.writerow(["Q50", est_size_50q])
+        writer.writerow(["Q75", est_size_75q])
+    print("Wrote particle size distribution summary to", output_path_distr_txt)
+    output_path_distr_img = os.path.join(output_folder,"size_distribution_"+timestr+".png")
+    print("Wrote plot of particle size distribution to", output_path_distr_img)
+    pl.savefig(output_path_distr_img)
+    pl.close()
+
+
+    #################################
+    # Confidence distribution histogram
+    #################################
+    output_path_conf_img = os.path.join(output_folder, "confidence_distribution_" + timestr + ".png")
+    output_path_conf_raw = os.path.join(output_folder,
+                                        "confidence_distribution_raw_" + timestr + ".txt")
+    output_path_conf_sum = os.path.join(output_folder,
+                                        "confidence_distribution_summary_" + timestr + ".txt")
+
+    # RAW DATA
+    np.savetxt(output_path_conf_raw,confidence,fmt='%1.2f')
+
+    # SUMMARY
+    conf_mean = np.mean(confidence)
+    conf_sd = np.std(confidence)
+    conf_25q = np.percentile(confidence, 25)
+    conf_50q = np.percentile(confidence, 50)
+    conf_75q = np.percentile(confidence, 75)
+
+    with open(output_path_conf_sum, 'w', newline='') as file:
+        writer = csv.writer(file)
+        writer.writerow(["Metric", "Value"])
+        writer.writerow(["MEAN", conf_mean])
+        writer.writerow(["SD", conf_sd])
+        writer.writerow(["Q25", conf_25q])
+        writer.writerow(["Q50", conf_50q])
+        writer.writerow(["Q75", conf_75q])
+
+    # PLOT
+    width = max(10, int((np.max(confidence) - np.min(confidence)) / 0.05))
+    pl.hist(confidence, bins=width)
+    pl.title("Confidence distribution")
+    bin_size_str = "{0:.2f}".format(((np.max(confidence) - np.min(confidence))/width))
+    pl.xlabel("Confidence (Bin size: " + bin_size_str + ")")
+    pl.ylabel("Count")
+    pl.savefig(output_path_conf_img)
+    print("")
+    print("## Particle confidence distribution ##")
+    print("Wrote confidence distribution summary", output_path_conf_sum)
+    print("Wrote confidence distribution to", output_path_conf_img)
+    print("")
+    print("#####################################")
+    pl.close()
+
 def prediction_result_to_disk(outdir, picking_results):
     for box_to_write in picking_results:
 
@@ -756,22 +901,32 @@ def rescale(box, image_height, image_width, export_size=None):
         boxwidth_in_pxl = export_size
     box.x = x_ll
     box.y = y_ll
+
     box.w = boxwidth_in_pxl
     box.h = boxheight_in_pxl
 
     return box
 
 
-def picking_postprocessing(picking_results, export_size, min_distance):
+def picking_postprocessing(picking_results, export_size, min_distance, min_size=None, max_size=None):
 
     # Rescaling
     for picking_result_micrograph in picking_results:
-        # if num_patches > 1:
-        #    image_width = int(picking_result_micrograph["img_width"]/num_patches)+2*overlap_patches
-        #    image_height = int(picking_result_micrograph["img_height"]/num_patches)+2*overlap_patches
-        # else:
+
         image_width = picking_result_micrograph["img_width"]
         image_height = picking_result_micrograph["img_height"]
+
+        # Save estimated box size in meta data
+
+        for box in picking_result_micrograph["boxes"]:
+            box.meta["boxsize_estimated"] = (
+                int(box.w * image_width), int(box.h * image_height))
+
+        for box in picking_result_micrograph["boxes_unfiltered"]:
+            box.meta["boxsize_estimated"] = (
+                int(box.w * image_width), int(box.h * image_height))
+
+        # Resize box
         picking_result_micrograph["boxes"] = [
             rescale(box, image_height, image_width, export_size)
             for box in picking_result_micrograph["boxes"]
@@ -792,6 +947,9 @@ def picking_postprocessing(picking_results, export_size, min_distance):
             picking_result_micrograph["boxes"] = min_distance_filter(
                 picking_result_micrograph["boxes"], min_distance
             )
+            picking_result_micrograph["boxes_unfiltered"] = min_distance_filter(
+                picking_result_micrograph["boxes_unfiltered"], min_distance
+            )
 
         num_remain_particles = np.sum([len(res["boxes"]) for res in picking_results])
         del_min_distance = sum_particles_before_filtering - num_remain_particles
@@ -817,7 +975,18 @@ def picking_postprocessing(picking_results, export_size, min_distance):
         for index in sorted(boxes_to_delete, reverse=True):
             del boxes[index]
 
-    return del_not_fully_immersed, del_min_distance
+    # Filtering according size
+    out_of_size = 0
+    for picking_result_micrograph in picking_results:
+
+        boxes = picking_result_micrograph["boxes"]
+        out_of_size_box_indices = get_out_of_sizes_boxes(boxes,min_size=min_size,max_size=max_size)
+
+        for index in sorted(out_of_size_box_indices, reverse=True):
+            del boxes[index]
+        out_of_size += len(out_of_size_box_indices)
+
+    return del_not_fully_immersed, del_min_distance, out_of_size
 
 
 def do_prediction(
@@ -855,7 +1024,7 @@ def do_prediction(
     :param yolo:
     :param otf: on the fly picking
     :param montior: Monitor input folder
-    :param write_direct: direct wiring
+    :param write_direct: on the fly writing of box files
     :return:
     """
     for path in input_path:
@@ -1000,31 +1169,7 @@ def do_prediction(
         #
         # TODO: Deprecated, anchors are now saved in the model and always defined.
         #
-
-        if len(config["model"]["anchors"]) > 2:
-            # general model is used
-            img_width = 4096.0 / num_patches
-            img_height = 4096.0 / num_patches
-            cell_w = img_width / grid_w
-            cell_h = img_height / grid_h
-            anchors = np.array(config["model"]["anchors"], dtype=float)
-            anchors[::2] = anchors[::2] / cell_w
-            anchors[1::2] = anchors[1::2] / cell_h
-
-        else:
-            # specifc model is used
-
-            img_width = float(img_first.shape[1]) / num_patches
-            img_height = float(img_first.shape[0]) / num_patches
-            cell_w = img_width / grid_w
-            cell_h = img_height / grid_h
-
-            box_width, box_height = config_tools.get_box_size(config)
-
-            anchor_width = 1.0 * box_width / cell_w
-            anchor_height = 1.0 * box_height / cell_h
-
-            anchors = [anchor_width, anchor_height]
+        anchors = utils.get_anchors(config, image_size=img_first.shape)
         print("Calculated Anchors using first image", anchors)
     else:
         print("Read Anchor from model", anchors)
@@ -1134,6 +1279,7 @@ def do_prediction(
     skipped_images = []
     sum_del_not_fully_immersed = 0
     sum_del_min_distance = 0
+    sum_del_out_of_size = 0
 
     do_picking = True
     while do_picking:
@@ -1158,6 +1304,7 @@ def do_prediction(
                             )
 
                             if resize_to is not None:
+                                print("Resize!")
                                 from PIL import Image
 
                                 image = np.array(
@@ -1292,15 +1439,18 @@ def do_prediction(
                                         export_size = config_tools.get_box_size(config)[
                                             0
                                         ]
-                                        del_not_fully_immersed, del_min_distance = picking_postprocessing(
+                                        del_not_fully_immersed, del_min_distance, del_min_size = picking_postprocessing(
                                             picking_results=[box_to_write],
                                             export_size=export_size,
                                             min_distance=kwargs["min_distance"],
+                                            min_size= kwargs["min_size"],
+                                            max_size=kwargs["max_size"]
                                         )
                                         sum_del_min_distance += del_min_distance
                                         sum_del_not_fully_immersed += (
                                             del_not_fully_immersed
                                         )
+                                        sum_del_out_of_size += del_min_size
                                         prediction_result_to_disk(
                                             kwargs["outdir"], [box_to_write]
                                         )
@@ -1314,6 +1464,8 @@ def do_prediction(
                         img_tiles = None
                         num_written_tiles = 0
                         image_indices = []
+                else:
+                    print("Not a valid image:", img_pth)
             else:
                 print("Not a valid image:", img_pth)
 
@@ -1357,17 +1509,53 @@ def do_prediction(
             "particles as they were not fully immersed in the micrograph",
         )
 
+        print(
+            "Deleted",
+            sum_del_out_of_size,
+            "particles as they were not out of specified size range",
+        )
+
     return boxes_to_write
 
 
+def get_out_of_sizes_boxes(boxes,min_size=None,max_size=None):
+    if min_size is None and max_size is None:
+        return []
+
+    out_of_size_boxes = []
+    for box_index, box in enumerate(boxes):
+        box_width = box.meta["boxsize_estimated"][0]
+        box_height = box.meta["boxsize_estimated"][1]
+        size = (box_width+box_height)/2
+        out_of_size = False
+
+        if min_size:
+            if size <= min_size:
+                out_of_size = True
+
+        if max_size:
+            if size >= max_size:
+                out_of_size = True
+
+        if out_of_size:
+            out_of_size_boxes.append(box_index)
+
+    return out_of_size_boxes
+
+
 def get_not_fully_immersed_box_indices(boxes, image_height, image_width):
     boxes_to_delete = []
     for box_index, box in enumerate(boxes):
+        box_width = box.w
+        box_height = box.h
+        if box_width == 0 and box_height == 0:
+            box_width = box.meta["boxsize_estimated"][0]
+            box_height = box.meta["boxsize_estimated"][1]
         if (
-            (box.x + box.w) >= image_width
+            (box.x + box_width) >= image_width
             or box.x < 0
             or box.y < 0
-            or (box.y + box.h) >= image_height
+            or (box.y + box_height) >= image_height
         ):
             boxes_to_delete.append(box_index)
     return boxes_to_delete
@@ -1419,13 +1607,14 @@ def filter_images_lowpass(
         filtered_images = pool.starmap(
             lowpass.filter_single_image, arg_tubles, chunksize=1
         )
+
     else:
         filtered_images = pool.starmap(
             lowpass.filter_single_image_and_write_to_disk, arg_tubles, chunksize=1
         )
     pool.close()
     pool.join()
-
+    filtered_images = [img for img in filtered_images if img is not None]
     return filtered_images
 
 
